#!/bin/bash
#
#                                       m   
#  mmmm   m   m  mmmm   mmmm    mmm   mm#mm 
#  #" "#  #   #  #" "#  #" "#  #"  #    #   
#  #   #  #   #  #   #  #   #  #""""    #   
#  ##m#"  "mm"#  ##m#"  ##m#"  "#mm"    "mm 
#  #             #      #                   
#  "             "      "                   
# This file is managed by puppet.  Do not make local changes.
# corresponds to lp:~lamont/launchpad-buildd/chroot-scripts rev 53

# Copyright 2009-2013 Canonical Ltd.
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.

# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.

set -e  #Abort on Shenangians.

function addmount {
    if ! grep -q "^$1[[:space:]]*$2" /etc/fstab; then
	echo "$@" >> /etc/fstab
    fi
    mount $2 || true
}

function resolvedir {
    # usage: resolvedir CHROOTBASE /DIRECTORY
    BASEDIR=$(cd $1 && /bin/pwd)
    DIR="${BASEDIR}$2"
    if [ -L "$DIR" ]; then
    	TARGET=$(readlink "$DIR")
	if [ ${TARGET#/} != ${TARGET} ]; then
	    resolvedir "${BASEDIR}" "$TARGET"
	else
	    resolvedir "${BASEDIR}" "${2%/*}$TARGET"
	fi
    else
    	echo "$DIR"
    fi
}

function usage {
cat <<EOF
Usage:
 $(basename $0) -d <distro> [-n|--name <name>] [-a|--arch <arch>] [-p|--ports]
       [--install "<package1> ..."] [--build-dep "<package1> ..."]
       [--binary <url>] [--source <url>] [--startdist <distro>]
       [--rapt]
       [--home]
       [--no-schroot]
       [--buildd] [--user <username>] [--noarchuser] [--group <groupname>] [--oem] [--live] [--nomangle] [--cat]
       [--lp]
       [--debian]
       [-m|--multiarch]
       [--help]
EOF
# we specifically do not mention --chdir and --chname, since they're frequently
# "absolutely not working correctly" - specifically, review especially fstab if used.
}
function help {
usage
echo
cat << EOF
EXAMPLES:  (without the weight of history)

"simple" schroot for services          <-- THIS IS QUITE POSSIBLY THE ONE YOU WANT 
      ./make-chroot.sh -d \$SUITE

"simple" schroot for services with /home bind mounted. <-- OR MAYBE THIS ONE
      ./make-chroot.sh -d \$SUITE --home

PORTER THINGS:
   porter chroot (with /home bind mounted, of course):
     ./make-chroot.sh -d \$SUITE --rapt
      you may also want to have -a \$ARCH and/or -n somename and etc: -d lucid -a i386 -n foo will create lucid-foo-i386

   i386 and amd64 porter chroots on an amd64 box:
      ./make-chroot.sh -d \$SUITE --rapt -a amd64 && ./make-chroot.sh -d \$SUITE --rapt -a i386
      This will create the \$SUITE-amd64 and \$SUITE-i386 porter schroots

LIVEFS/DAK:
   livecd chroot for \$SUITE:
      ./make-chroot.sh -d \$SUITE --live
      For an architecture other than the native one, add: -a \$DPKG_ARCHITECTURE --noarchuser

   dak chroot for \$SUITE-cat-proposed:
      ./make-chroot.sh -d \$SUITE -n cat-proposed --buildd --cat

LAUNCHPAD:
   launchpad chroot tarball (for the architecture of the machine you are on):
      ./make-chroot.sh -d \$SUITE --lp
EOF
}

deburl=""
srcurl=""
chrootdir=""
livecd=""
buildduser=buildd
builddgroup=buildd
xdebian=""
pkgmangle="yes"
debootstrap_args=""
multiarch=""
schroot=yes
schroot_used=""
home=""

while [ $# -gt 0 ]
do
    case "$1" in
	    -v|--verbose)	set -x; shift;;

	    -d|--distro)	distro=$2; shift; shift;;
	    --startdist)	startdist=$2; shift; shift;;
	    -n|--name)		name=$2; shift; shift;;
	    -a|--arch)		arch=$2; shift; shift;;
 	    -p|--ports)		ports=-ports; shift;;

	    --binary)		deburl=$2; shift; shift;;
	    --source)		srcurl=$2; shift; shift;;

	    --install)		installs=$2; test -n "$installs" || installs=" "; shift; shift;;
	    --build-dep)	builddep=$2; test -n "$builddep"  || builddep=" "; shift; shift;;

	    --buildd)		buildd=yes; schroot=""; shift;;
	    --oem)		buildd=yes; schroot=""; oem=yes; shift;;			# vs standard archive
 	    --nomangle)		pkgmangle=""; shift;;
	    --debian)		debian=yes; shift;;				# building a debian chroot
 	    --xdebian)		debian=yes; xdebian=yes; shift;;		# cross-built debian
	    --user)		buildduser=$2; shift; shift;;
	    --noarchuser)	noarchuser=yes; shift;;
	    --group)		builddgroup=$2; shift; shift;;
	    --cat)		cat=yes; pkgmangle=""; shift;;			# cat chroot
	    -m|--multiarch)	multiarch=yes; shift;;				# keep multiarch configs

	    --lp|--launchpad)	launchpad=yes; chrootdir=chroot-autobuild; buildd=yes; schroot=""; shift;;
	    --debootstrap-args)	debootstrap_args="$2"; shift; shift;;

	    --schroot)		schroot_used=yes; schroot=yes; shift;;
	    --no-schroot)	schroot=""; shift;;
	    --rapt)		rapt=yes; home=yes; shift;;
	    --home)		home=yes; shift;;
	    --no-home)		home=""; shift;;
	
	    --chdir)		chrootdir=$2; shift; shift;;
	    --chname)		chrootname=$2; shift; shift;;

	    -h|--help|--herb)	help; exit 0;;
	    --live)		buildd=yes; schroot=""; livecd=-live; shift;;
	    *)			usage; exit 1;;
    esac
done

# We must have a distro name
if test -z "$distro"; then
	usage
	exit 2
fi
if test -n "$dchroot" && test -n "$buildd"; then
	echo cannot use --dchroot and --buildd together. >&2
	exit 2
fi
if test -n "$schroot_used" && test -n "$buildd"; then
	echo cannot use --schroot and --buildd together. >&2
	exit 2
fi

if test "$(id -u)" != 0; then
	echo "you must be root to build chroots" >&2
	exit 2
fi

if test -n "$dchroot" && test ! -w /etc/dchroot.conf; then
	echo "/etc/dchroot.conf does not exist. install dchroot" >&2
	exit 2
fi
if test -n "$schroot"; then
	if test ! -w /etc/schroot/schroot.conf; then
		echo "/etc/schroot/schroot.conf does not exist. install schroot" >&2
		exit 2
	fi
	if test -f /etc/schroot/chroot.d/$chrootname; then
		echo "/srv/schroot/chroot.d/$chrootname already exists, not overwriting it" >&2
	fi
fi

# set defaults and figure out the chroot name.
test -n "$startdist" || startdist=$distro

if test -z "$chrootname"; then
	if test -n "$buildd" -a -n "$cat" -a -z "$name"; then
		test $distro = dapper && name=cat || name=cat-proposed
	fi
	test -n "$name" && chrootname=$distro-$name || chrootname=$distro
fi

suite=$chrootname

BASEDIR=/srv/chroots
archarg=""
test -z "$arch" || archarg="--arch $arch"
ext=""
if test -n "$buildd"; then
	if test -n "$arch" && test -z "$noarchuser"; then
		buildduser=$buildduser-$arch
	fi
	if test -z "$debian" || test -n "$xdebian"; then
		ext=-$suite$livecd
	fi
	BASEDIR=$(eval echo ~$buildduser)/build$ext
	UPLOADDIR=$(eval echo ~$buildduser)/upload$ext
	chrootname="chroot-$suite"
	rapt=""

	# create the build and upload directories for this suite.
	if test -z "$launchpad"; then
		install -d -o$buildduser -g$builddgroup -m755 $BASEDIR $UPLOADDIR
	fi
else
	test -z "$arch" || chrootname=$suite-$arch
fi

# we'll use $arch later, so figure out where we are.
if test -z "$arch"; then
	case $(uname -m) in
		x86_64)		arch=amd64;;
		i?86)		arch=i386;;
		parisc*)	arch=hppa;;
		sparc*)		arch=sparc;;
		ppc64)		[ $(dpkg --print-architecture) = powerpc ] && arch=powerpc || arch=ppc64;;
		ppc*)		arch=powerpc;;
		arm*)		arch=armel;;
		*)		arch=$(uname -m);;
	esac
fi

if test -n "$buildd" && test -z "$launchpad" && ! id "$buildduser" >/dev/null 2>&1; then
	# we're missing $buildduser and not doing launchpad-ish stuff.
	echo user "$buildduser" does not exist >&2
	exit 1
fi

test -n "$chrootdir" || chrootdir=$BASEDIR/$chrootname
if test -d $chrootdir && [ -n "$(ls $chrootdir)" ]; then
	echo $chrootdir exists >&2
	exit 1
fi
mkdir -p $chrootdir

if test -z "$installs"; then
	installs="gnupg libnss-db zsh fakeroot build-essential devscripts nvi vim
	          ccache less devscripts bzr git-core patchutils devscripts-el
	          gdb"
fi

# figure out whether or not we should be using --ports
case "$distro:$arch" in
	*:armel)	ports=-ports;;
	*:armhf)	ports=-ports;;
	*:hppa)		ports=-ports;;
	*:ia64)		ports=-ports;;
	*:lpia)		ports=-ports;;
	dapper:sparc)	;;
	*:sparc)	ports=-ports;;
	warty:powerpc)		;;
	hoary:powerpc)		;;
	breezy:powerpc)		;;
	dapper:powerpc)		;;
	*:powerpc)	ports=-ports;;
esac

if test -z "$deburl"; then
	case ${debian}:$(hostname --fqdn) in
		yes:*)	deburl=http://ftp.uk.debian.org/debian;;
		*.buildd|*.ppa)
			deburl=http://ftpmaster.internal/ubuntu;;
		*)	case $suite in
			    warty*|hoary*|breezy*|edgy*|feisty*|gutsy*|hardy*|intrepid*|jaunty*|karmic*|maverick*|natty*)
				deburl=http://old-releases.ubuntu.com/ubuntu;
				test -n "$srcurl" || srcurl=$deburl
				;;
			    *)
				if test -n "$ports"; then
					deburl=http://ports.ubuntu.com/ubuntu-ports
				else
					deburl=http://archive.ubuntu.com/ubuntu
				fi
			esac
			;;
	esac
fi

if test -z "$srcurl"; then
	case ${debian}:$(hostname --fqdn) in
		yes:*)		srcurl=http://ftp.uk.debian.org/debian;;
		*.buildd|*.ppa) srcurl=http://ftpmaster.internal/ubuntu;;
		*)		srcurl=http://archive.ubuntu.com/ubuntu;;
	esac
fi

CHROOT=chroot
STAGE2=""
case "$arch:$(uname -m)" in
	i386:x86_64)	LINUX32="linux32"; CHROOT="linux32 chroot" ;;
	lpia:x86_64)	LINUX32="linux32"; CHROOT="linux32 chroot" ;;
	ppc64:ppc64)	;;
	powerpc:ppc64)	LINUX32="linux32";;
	*:parisc64)	LINUX32="linux32";;
	*:sparc64)	LINUX32="linux32";;
	armel:i386)	STAGE2=/usr/bin/qemu-arm-static;;
	armel:x86_64)	STAGE2=/usr/bin/qemu-arm-static;;
	armhf:i386)	STAGE2=/usr/bin/qemu-arm-static;;
	armhf:x86_64)	STAGE2=/usr/bin/qemu-arm-static;;
	powerpc:i386)	STAGE2=/usr/bin/qemu-ppc-static;;
	powerpc:x86_64)	STAGE2=/usr/bin/qemu-ppc-static;;
	*)		;;
esac

# HACK ALERT:
#   PPC lucid and later on pre 2.6.17 kernels need to have /proc mounted,
#   or they fail.  See also $MOUNTED_PROC  " " gets us past the -n, does no copies
#   and is used as a flag later to tell that we mounted /proc.
if dpkg --compare-versions $(uname -r) lt 2.6.17 && dpkg --compare-versions $suite ge lucid; then
	STAGE2=" ";
fi;

if test -n "$STAGE2"; then
	debootstrap ${debootstrap_args} --variant=buildd --foreign $archarg $startdist $chrootdir $deburl
	for f in $STAGE2; do
		mkdir -p $chrootdir${f%/*} && rsync -a $f $chrootdir${f%/*}
	done
	if test "$STAGE2" = " "; then
		mount -t proc proc $chrootdir/proc
	fi
	$CHROOT $chrootdir debootstrap/debootstrap --second-stage
else
	debootstrap ${debootstrap_args} --variant=buildd $archarg $startdist $chrootdir $deburl
fi

# We don't want multiarch by default in chroots, despite it being on by
# default in Ubuntu's amd64 port.  In the future, this may change, and
# we might even want multiarch to be configurable when building a chroot
if test -z "$multiarch"; then
	rm -f ${chrootdir}/etc/dpkg/dpkg.cfg.d/multiarch
fi

exec 6> $chrootdir/etc/apt/sources.list
if test -n "$buildd"; then
	# --debian changes this below.
	cat <<- EOF > $chrootdir/etc/apt/apt.conf.d/99buildd
		APT::Get::AllowUnauthenticated "1";
		APT::Install-Recommends "0";
	EOF
	if test -n "$debian"; then
		echo 'APT::Install-Recommends False;' > $chrootdir/etc/apt/apt.conf.d/99buildd
		cat <<- EOF >&6
			deb $deburl $distro main
		EOF
	elif test -n "$oem"; then
		cat <<- EOF >&6
		#deb https://buildd:SECRET@cesg.canonical.com/canonical $suite public private
		#deb https://buildd:SECRET@cesg.canonical.com/canonical $suite-devel public private
		#deb-src https://buildd:SECRET@cesg.canonical.com/canonical $suite public private
		
		deb $deburl $distro main restricted universe multiverse
		EOF
	elif test -n "$cat"; then
		cat <<- EOF >&6
		deb $deburl $distro main restricted universe multiverse
		deb $deburl $distro-updates main restricted universe multiverse
		deb $deburl $distro-security main restricted universe multiverse
		deb http://archive.admin.canonical.com/ubuntu $distro-cat main
		deb http://archive.admin.canonical.com/ubuntu $suite main
		deb-src http://archive.admin.canonical.com/ubuntu $suite main
		EOF
	else
		# ubuntu, non-OEM chroot.
		COMP="main"
		case $startdist in
			[a-k]*) COMP="main universe";;  # we need universe for policyrcd-script-zg2
		esac
		cat <<- EOF >&6
			deb $deburl $distro $COMP
		EOF
	fi
else
	if test -z "$debian"; then
		cat <<- EOF >&6
		deb $deburl $distro main restricted universe multiverse
		deb $deburl $distro-updates main restricted universe multiverse
		deb $deburl $distro-security main restricted universe multiverse
		deb-src $srcurl $distro main restricted universe multiverse
		deb-src $srcurl $distro-updates main restricted universe multiverse
		deb-src $srcurl $distro-security main restricted universe multiverse
		EOF
	else
		cat <<- EOF >&6
		deb $deburl $distro main
		deb-src $srcurl $distro main
		EOF
	fi
fi

# schroot chroots get /home, schroot, and -cat
if test -n "$schroot";  then
	case $suite in
		dapper*)	catsuite=dapper-cat;;
		edgy*)		catsuite=edgy-cat;;
		[h-k]*)		catsuite=hardy-cat;;	# yes: do not care about hoary.
		[l-o]*)		catsuite=lucid-cat;;
		*)		catsuite=precise-cat;;
	esac
	if test "$arch" != "lpia"; then
	    echo deb http://archive.admin.canonical.com/ubuntu $catsuite main
	else
	    # lpia lacks *-cat, so abuse i386... although it means unauthenticated pacakges
	    echo deb http://archive.admin.canonical.com/ubuntu/ dists/$catsuite/main/binary-i386/
	fi >&6
	if test -n "$cat"; then
	    echo deb-src http://archive.admin.canonical.com/ubuntu $catsuite main
	fi
	cat <<- EOF > $chrootdir/etc/apt/preferences
		Package: *
		Pin: release a=$catsuite
		Pin-Priority: 50
	EOF

	if [ -n "$home" ]; then
	    addmount /home      $chrootdir/home     none       bind     0  0
	fi
	if [ -n "$schroot" ] && [ ! -f /etc/schroot/chroot.d/$chrootname ]; then
		cat << EOF > /etc/schroot/chroot.d/$chrootname
[$chrootname]
description=$chrootname porter chroot
directory=$chrootdir
root-groups=adm
groups=warthogs,porting_team
EOF
		if [ -n "$LINUX32" ]; then
			echo "personality=$LINUX32" >> /etc/schroot/chroot.d/$chrootname
		fi
	fi
	if [ -w /etc/dchroot.conf ] && ! grep -q "$chrootdir$" /etc/dchroot.conf; then
		if dchroot --version >/dev/null 2>&1; then
			sed -i "1i\
$chrootname $chrootdir $LINUX32" /etc/dchroot.conf
		else
			sed -i "1i\
$chrootname $chrootdir" /etc/dchroot.conf
		fi
	fi
fi
exec 6>&-

MOUNTED_PROC=""
# if we're not building chroot-autobuild for a release, give us proc and devpts and shm
if test -z "$launchpad"; then
	if test -z "$buildd" || test -n "$noarchuser"; then
		addmount none  $chrootdir/proc     proc       defaults 0  0
		addmount none  $(resolvedir $chrootdir /dev/pts)  devpts     defaults 0  0
		addmount none  $(resolvedir $chrootdir /dev/shm)  tmpfs      defaults 0  0
	else
		addmount none  $chrootdir/proc     proc       defaults 0  0
		addmount none  $(resolvedir $chrootdir /dev/pts)  devpts     defaults 0  0
		addmount none  $(resolvedir $chrootdir /dev/shm)  tmpfs      defaults 0  0
	fi
	MOUNTED_PROC=yes
fi

# everyone gets invoke-rc.d (and later initctl) overridden.
mkdir -p $chrootdir/usr/local/sbin
cat <<- EOF > $chrootdir/usr/local/sbin/policy-rc.d
	#!/bin/sh

	# policy-rc.d script for chroots.
	# Copyright (c) 2007 Peter Palfrader <peter@palfrader.org>
	# License: <weasel> MIT, if you want one.

	while true; do
	  case "\$1" in
	    -*)         shift ;;
	    makedev)    exit 0;;
	    *) echo "Not running services in chroot."; exit 101 ;;
	  esac
	done
EOF
chmod 755 $chrootdir/usr/local/sbin/policy-rc.d

if test ! -f $chrootdir/etc/hosts; then
	cat <<- EOF > $chrootdir/etc/hosts
		127.0.0.1	localhost.localdomain	localhost

		# The following lines are desirable for IPv6 capable hosts
		::1     ip6-localhost ip6-loopback
		fe00::0 ip6-localnet
		ff00::0 ip6-mcastprefix
		ff02::1 ip6-allnodes
		ff02::2 ip6-allrouters
		ff02::3 ip6-allhosts
	EOF
	chmod 644 $chrootdir/etc/hosts
	if test -n "$buildd"; then
		echo 127.0.1.1 $(hostname --fqdn) $(hostname --short) >> $chrootdir/etc/hosts
	fi
fi
# Put together the bootstrap script to run inside the chroot.
exec 5>$chrootdir/root/bootstrap
cat >&5 <<- EOBOOT
	#!/bin/bash
	set -e
	export HOME=/root

	# divert initctl
	dpkg-divert --local --rename --divert /sbin/initctl.real /sbin/initctl
	ln -s /bin/true /sbin/initctl

	# we neeed things in /dev for the compiler packages, at least
	test -e /dev/fd      || ln -sf /proc/self/fd /dev/fd
	test -f /dev/stdin   || ln -sf /proc/self/fd/0 /dev/stdin
	test -f /dev/stdout  || ln -sf /proc/self/fd/1 /dev/stdout
	test -f /dev/stderr  || ln -sf /proc/self/fd/2 /dev/stderr

	mkdir -p etc/apt/preferences.d etc/apt/sources.list.d	# newer apt wants these
	apt-get -qq update
	apt-get -y -q --force-yes install policyrcd-script-zg2
EOBOOT
if test -n "$buildd" ; then
	# non-interactive frontend for buildds
	cat >&5 <<- EOF
		if ! echo SET debconf/frontend Noninteractive | debconf-communicate debconf; then
			echo man-db man-db/auto-update boolean false | debconf-set-selections #RT36503
		fi
		if ! echo SET shared/accepted-sun-dlj-v1-1 true | debconf-communicate debconf; then
			echo sun-java6-bin shared/accepted-sun-dlj-v1-1 boolean true | debconf-set-selections  #RT37633
			echo sun-java6-jre shared/accepted-sun-dlj-v1-1 boolean true | debconf-set-selections  #RT37633
		fi
	EOF
	test -n "$debian" -a -n "$cat"  || echo "sed -i 's/ universe//' /etc/apt/sources.list || true" >&5
	echo apt-get -qq update >&5
else
	echo apt-get -y -q --force-yes install gnupg >&5
fi
	echo apt-get -y -q dist-upgrade >&5

if test -n "$buildd"; then
	if test -z "$livecd"; then
	    echo 'echo INVALID > /etc/hostname' >&5
	else
	    echo echo $(hostname --short) ' > /etc/hostname' >&5
	fi

	# make sure $buildduser exists
	if ! id $buildduser >/dev/null 2>&1; then
		cat >&5 <<- EOF
			test -x /usr/sbin/adduser  || ADDUSER=yes
			test -z "\$ADDUSER"  || apt-get install -y adduser
			if ! getent group $builddgroup >/dev/null 2>&1; then
			    addgroup $builddgroup --gid 2501
			fi
			adduser --disabled-password --gecos 'buildd user' --uid 2001 --ingroup $builddgroup $buildduser
			test -z "\$ADDUSER" || apt-get remove -y --purge adduser
		EOF
	fi
	for file in passwd shadow; do
		if ! grep -q ^$buildduser: $chrootdir/etc/$file; then
			getent $file $buildduser | sed -n 's/^\([^:]*\):[^:]*:/\1:x:/p' >> $chrootdir/etc/$file
		fi
	done

	for file in group; do
		if ! grep -q ^$builddgroup: $chrootdir/etc/$file; then
			getent $file $builddgroup | sed -n 's/^\([^:]*\):[^:]*:/\1:x:/p' >> $chrootdir/etc/$file
		fi
	done

	for dir in var/debbuild/ var/debbuild/srcdep-lock home/$buildduser build/$buildduser \
		   home/$buildduser/.ccache; do
		echo install -d -m775 -g$builddgroup -o$buildduser /$dir >&5
	done
	for file in var/debbuild/avg-build-time var/debbuild/avg-build-space; do
		echo install -m664 -g$builddgroup -o$buildduser /dev/null /$file >&5
	done
	if test -z "$debian" || test -n "$xdebian"; then
		echo install -m664 -g$builddgroup -o$buildduser /dev/null /CurrentlyBuilding >&5
	fi

	# at one point, kernel builds needed this
	echo do_initrd = Yes > $chrootdir/etc/kernel-img.conf
	# make it a dummy fstab, for real this time
	echo /dev/root / ext2 noatime,errors=remount-ro 0 1 > $chrootdir/etc/fstab
	chmod 644 $chrootdir/etc/fstab

	if test -z "$debian" && test -n "$pkgmangle"; then
		echo 'apt-get install -y pkgbinarymangler || true' >&5
		PKG=/etc/pkgbinarymangler
		if test -n "$oem"; then
			echo "sed -i /^enable:/s/true/false/ ${PKG}/striptranslations.conf || true"
			echo "apt-get remove --purge pkg-create-dbgsym || true"
			echo "apt-get install -y apt-transport-https || true"
		elif test -n "$launchpad"; then
			echo "sed -i /^enable/s/false/true/ ${PKG}/maintainermangler.conf || true"
			echo "sed -i /^invalid_current/s/ignore/fail/ ${PKG}/maintainermangler.conf || true"
			echo "sed -i /^enable/s/false/true/ ${PKG}/striptranslations.conf || true"
			echo "sed -i /^invalid_current/s/ignore/fail/ ${PKG}/striptranslations.conf || true"
			echo "apt-get install -y pkg-create-dbgsym || true"
			echo "apt-get install -y apt-transport-https || true"
		fi >&5
	fi
	echo apt-get install -y fakeroot build-essential >&5	# insist on success
	test -n "$livecd" && installs="livecd-rootfs ubuntu-defaults-builder" || installs=""
	if test -n "$livecd" && test "$arch" =  "i386"; then
		installs="$installs ltsp-server"
	fi
	builddep=""
else	# not a buildd
	echo "$chrootname" > $chrootdir/etc/debian_chroot
	cp -a /etc/nsswitch.conf $chrootdir/etc/nsswitch.conf
fi
for pkg in $installs; do
	echo "apt-get -y install $pkg || true" >&5	# do not care if these install
done
for pkg in $builddep; do
	echo "apt-get -y build-dep $pkg || true" >&5	# do not care if these install
done
if test -n "$rapt"; then
	echo "apt-get -y -oAPT::Get::AllowUnauthenticated=true install sudo rapt libnss-db" >&5		# insist on success
fi

if test -z "$buildd"; then
	# if the real /etc is in bzr, then the inner one should be too.
	if test -d /etc/.bzr; then
		cat >&5 <<- EOBOOT
			apt-get install -y bzr || true
			if test -x /usr/bin/bzr && ! test -d /etc/.bzr; then
			  cd /etc
			  bzr init
			  chgrp -R adm .bzr
			  chmod -R g+w,o-rwx .bzr
			  find .bzr -type d | xargs chmod g+s
			  bzr ignore ld.so.cache
			  bzr ignore adjtime
			  bzr ignore '*-'
			  bzr ignore '*.dpkg-new'
			  bzr add
			  bzr whoami --branch "Root <root@localhost>"
			  bzr commit -m 'Initial import of /etc.'
			fi
		EOBOOT
	fi
fi
exec 5>&-

if test "$STAGE2" = " "; then
	mount -t proc proc $chrootdir/proc
fi
$CHROOT $chrootdir bash /root/bootstrap || true
if test "$STAGE2" = " " && test -z "$MOUNTED_PROC"; then
	umount $chrootdir/proc || true
fi

if test -z "$launchpad"; then
	test -n "$buildd" || ud-replicate || true
	if test -z "$buildd" &&  test -n "$rapt"; then
		echo '%warthogs ALL=(root) NOPASSWD: /usr/local/bin/apt-get' >> $chrootdir/etc/sudoers
		mkdir -p $chrootdir/usr/local/bin/
		cat <<- EOF > $chrootdir/usr/local/bin/apt-get
		#!/bin/sh

		if [ -z \$SUDO_UID ]; then
		  exec /usr/bin/apt-get \$@
		else
		  exec /usr/sbin/rapt \$@
		fi
		EOF
		chmod 755 $chrootdir/usr/local/bin/apt-get
	fi
	if [ -n "$schroot" ] && [ -z "$home" ]; then
		echo "########################################################################################"
		echo "The chroot was created acccessible via schroot, but /home was not bind mounted."
		echo "If you want /home mounted, add the following to /etc/fstab and mount it:"
	        echo "/home      $chrootdir/home     none       bind     0  0"
		echo "########################################################################################"
	fi
else	# this is for launchpad, package it up.
	# the rm will hit a couple directories, we don't care
	rm $chrootdir/var/cache/apt/archives/* $chrootdir/var/lib/apt/lists/* 2>/dev/null || true
	echo creating chroot-ubuntu-$distro-$arch.tar.bz2
	tar cjf chroot-ubuntu-$distro-$arch.tar.bz2 $chrootdir
fi
