import os
import shlex
import yaml
import mojo


class DuplicateCharmError(Exception):
    def __init__(self, charm_name, source1, source2):
        super(DuplicateCharmError, self).__init__()
        self.message = "{} and {} are both named '{}'".format(charm_name, source1, source2)
        self.charm_name = charm_name
        self.sources = [source1, source2]


class InvalidCharmSpec(Exception):
    pass


class CharmRepo(object):
    def __init__(self, repo_dir):
        super(CharmRepo, self).__init__()
        self.repo_dir = repo_dir
        self.charms = {}

    def charm_metadata(self, charm_dir):
        md_file = os.path.join(charm_dir, 'metadata.yaml')
        with open(md_file) as f:
            return yaml.safe_load(f)

    def add_charm(self, charm_name, source):
        md_name = self.charm_metadata(source)['name']
        if md_name in self.charms:
            raise DuplicateCharmError(charm_name, source, self.charms[md_name])
        self.charms[md_name] = source
        os.symlink(source, os.path.join(self.repo_dir, charm_name))

    def build_from_config(self, charm_config, build_dir):
        """
        Build a charm repo from a charm config file.

        Lines beginning with "#" are comments. Empty lines are ignored.
        Configuration lines are either

            charm_name

        where the path inside build_dir is expected to be the same as the
        charm name or:

            charm_name path

        where path is relative to build_dir. Charms directories inside the
        repo are always named charm_name.
        """

        with open(charm_config) as f:
            for line in f:
                # Comment lines begin with #, empty lines are ignored
                if line.strip().startswith("#") or len(line.strip()) == 0:
                    continue
                args = shlex.split(line.strip())
                charm_name = args[0]
                if len(args) == 1:
                    # charm path is assumed to be same as the name
                    source = os.path.join(build_dir, args[0])
                elif len(args) == 2:
                    source = os.path.join(build_dir, args[1])
                else:
                    raise InvalidCharmSpec("Unrecognized format for line: {}".format(line))
                self.add_charm(charm_name, source)
