import argparse
import mojo
import os
import sys
import subprocess
from charm_repo import CharmRepo


def makedirs_or_exit(*dirs):
    for directory in dirs:
        if not os.path.isdir(directory):
            try:
                os.makedirs(directory)
            except OSError as e:
                print e.strerror, '({})'.format(e.filename)
                print "Initialization failed"
                sys.exit(1)


def get_dep_dir(depname):
    dep_dir = os.path.join(os.path.dirname(__file__), '..', 'deps', depname)
    if os.path.isdir(dep_dir):
        return dep_dir
    else:
        return None


# -----------------------
# commands
# -----------------------


def init(args):
    "Set up workspace and fetch a copy of the configuration"
    workspace = mojo.get_workspace(args.workspace)
    print "Initializing workspace"
    makedirs_or_exit(workspace)

    cmd = ["bzr", "branch"]
    cmd.append(args.spec_url[0])
    cmd.append(mojo.config_dir(workspace))
    try:
        subprocess.check_call(cmd)
    except subprocess.CalledProcessError as e:
        sys.exit(e.returncode)

    directories = (
        mojo.build_dir(workspace),
        mojo.charm_repo_dir(workspace),
        mojo.charm_repo_series_dir(workspace),
        mojo.log_dir(workspace),
    )
    makedirs_or_exit(*directories)


def run_script(section, workspace, envtag, critical=False):
    "Run a script portion of a deployment process"
    # XXX: Add support for chroot'd builds
    print "Running {} script".format(section)
    workspace = mojo.get_workspace(workspace)
    try:
        prog = mojo.get_config_file(workspace, section, envtag)
    except mojo.ConfigNotFoundException as e:
        print e.message
        if critical:
            sys.exit(1)
        else:
            return
    env = os.environ.copy()
    env['MOJO_BUILD_DIR'] = mojo.build_dir(workspace)
    env['MOJO_CONFIG_DIR'] = mojo.config_dir(workspace)
    env['MOJO_LOG_DIR'] = mojo.log_dir(workspace)
    env['MOJO_LOCAL_DIR'] = mojo.local_dir(workspace)
    env['MOJO_ENV_TAG'] = envtag
    os.chdir(env['MOJO_BUILD_DIR'])
    subprocess.check_call([prog], env=env)


def collect(args):
    "Collect build resources using config-manager"
    print "Building resource tree"
    workspace = mojo.get_workspace(args.workspace)
    try:
        config = mojo.get_config_file(workspace, "branches", args.envtag)
    except mojo.ConfigNotFoundException as e:
        print e.message
        sys.exit(1)
    cm_dir = get_dep_dir("config-manager")
    cm = os.path.abspath(os.path.join(cm_dir, 'cm.py'))
    os.chdir(mojo.build_dir(workspace))
    cmd = ['python', cm, '-v', 'update', config]
    subprocess.check_call(cmd)


def build(args):
    "Run deployment's build script"
    run_script("build", args.workspace, args.envtag)


def charm_repo(args):
    "Build a charm repository"
    workspace = mojo.get_workspace(args.workspace)
    repo_dir = mojo.charm_repo_series_dir(workspace)
    print "Building charm repository at {}".format(repo_dir)
    build_dir = mojo.build_dir(workspace)
    charm_repo_config = mojo.get_config_file(workspace, "charms", args.envtag)
    charm_repo = CharmRepo(repo_dir)
    charm_repo.build_from_config(charm_repo_config, build_dir)


def predeploy(args):
    "Run deployment's pre-deployment script"
    run_script("predeploy", args.workspace, args.envtag)


def services(args):
    "Build or update services"
    workspace = mojo.get_workspace(args.workspace)
    repo = mojo.charm_repo_dir(workspace)
    print "Deploying services"
    deploy_config = mojo.get_config_file(workspace, "services", args.envtag)
    deployer_dir = get_dep_dir("juju-deployer")
    deployer = os.path.abspath(os.path.join(deployer_dir, 'deployer.py'))
    env = os.environ.copy()
    env['PYTHONPATH'] = ":".join([get_dep_dir("websocket-client"),
                                  get_dep_dir("python-jujuclient")])
    # get the deployment name
    cmd = ['python', deployer, '-c', deploy_config, "-l"]
    deploy_name = subprocess.check_output(cmd, env=env).split("\n")[0]
    # run the deploy
    os.chdir(repo)
    cmd = ['python', deployer, '-c', deploy_config, "-d", deploy_name]
    subprocess.check_call(cmd, env=env)


def relations(args):
    "Set up the service relations"
    workspace = mojo.get_workspace(args.workspace)
    repo = mojo.charm_repo_dir(workspace)
    print "Setting up relations"
    deploy_config = mojo.get_config_file(workspace, "relations", args.envtag)
    deployer_dir = get_dep_dir("juju-deployer")
    deployer = os.path.abspath(os.path.join(deployer_dir, 'deployer.py'))
    env = os.environ.copy()
    env['PYTHONPATH'] = ":".join([get_dep_dir("websocket-client"),
                                  get_dep_dir("python-jujuclient")])
    # get the deployment name
    cmd = ['python', deployer, '-c', deploy_config, "-l"]
    deploy_name = subprocess.check_output(cmd, env=env).split("\n")[0]
    # run the deploy
    os.chdir(repo)
    cmd = ['python', deployer, '-c', deploy_config, "-d", deploy_name]
    subprocess.check_call(cmd, env=env)


def postdeploy(args):
    "Run deployment's post-deployment script"
    run_script("postdeploy", args.workspace, args.envtag)


def verify(args):
    "Verify the running service"
    run_script("verify", args.workspace, args.envtag, critical=True)

# -----------------------
# main
# -----------------------


def main():
    ap = argparse.ArgumentParser()
    ap.add_argument("--workspace", "-w", help="Workspace path (default: '.')",
                    default=None)
    ap.add_argument("--envtag", "-e", help="Environment tag")

    sp = ap.add_subparsers(title="subcommands")

    initcmd = sp.add_parser("init", help="Initialize a mojo workspace")
    initcmd.add_argument("spec_url", nargs=1, help="Configuration spec url (bzr)")
    initcmd.set_defaults(func=init)

    collectcmd = sp.add_parser("collect", help="Collect resources for build")
    collectcmd.set_defaults(func=collect)

    buildcmd = sp.add_parser("build", help="Execute project build scripts")
    buildcmd.set_defaults(func=build)

    charmscmd = sp.add_parser("charm-repo", help="Build charm repository")
    charmscmd.set_defaults(func=charm_repo)

    predeploycmd = sp.add_parser("predeploy", help="Execute project predeploy scripts")
    predeploycmd.set_defaults(func=predeploy)

    servicescmd = sp.add_parser("services", help="Deploy services")
    servicescmd.set_defaults(func=services)

    relationscmd = sp.add_parser("relations", help="Build relations between services")
    relationscmd.set_defaults(func=relations)

    postdeploycmd = sp.add_parser("postdeploy", help="Execute project postdeploy scripts")
    postdeploycmd.set_defaults(func=postdeploy)

    verifycmd = sp.add_parser("verify", help="Verify that the service is working")
    verifycmd.set_defaults(func=verify)

    args = ap.parse_args()
    args.func(args)
